@@ -412,7 +412,9 @@ def test_function_plain_field_serializer_to_python():
             core_schema.typed_dict_schema(
                 {
                     'x': core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.field_function_plain_ser_schema(Model.ser_x))
+                        core_schema.int_schema(
+                            serialization=core_schema.field_plain_serializer_function_ser_schema(Model.ser_x)
+                        )
                     )
                 }
             ),
@@ -426,7 +428,7 @@ def test_function_wrap_field_serializer_to_python():
     class Model:
         x: int
 
-        def ser_x(self, v: Any, serializer: core_schema.SerializeWrapHandler, _) -> str:
+        def ser_x(self, v: Any, serializer: core_schema.SerializerFunctionWrapHandler, _) -> str:
             x = serializer(v)
             assert self.x == 1_000
             return f'{x:_}'
@@ -438,7 +440,7 @@ def test_function_wrap_field_serializer_to_python():
                 {
                     'x': core_schema.typed_dict_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_function_wrap_ser_schema(
+                            serialization=core_schema.field_wrap_serializer_function_ser_schema(
                                 Model.ser_x, schema=core_schema.any_schema()
                             )
                         )
@@ -465,7 +467,9 @@ def test_function_plain_field_serializer_to_json():
             core_schema.typed_dict_schema(
                 {
                     'x': core_schema.typed_dict_field(
-                        core_schema.int_schema(serialization=core_schema.field_function_plain_ser_schema(Model.ser_x))
+                        core_schema.int_schema(
+                            serialization=core_schema.field_plain_serializer_function_ser_schema(Model.ser_x)
+                        )
                     )
                 }
             ),
@@ -479,7 +483,7 @@ def test_function_wrap_field_serializer_to_json():
     class Model:
         x: int
 
-        def ser_x(self, v: Any, serializer: core_schema.SerializeWrapHandler, _) -> str:
+        def ser_x(self, v: Any, serializer: core_schema.SerializerFunctionWrapHandler, _) -> str:
             assert self.x == 1_000
             x = serializer(v)
             return f'{x:_}'
@@ -491,7 +495,7 @@ def test_function_wrap_field_serializer_to_json():
                 {
                     'x': core_schema.typed_dict_field(
                         core_schema.int_schema(
-                            serialization=core_schema.field_function_wrap_ser_schema(
+                            serialization=core_schema.field_wrap_serializer_function_ser_schema(
                                 Model.ser_x, schema=core_schema.any_schema()
                             )
                         )
