@@ -180,7 +180,8 @@ all_errors = [
     ('recursion_loop', 'Recursion error - cyclic reference detected', None),
     ('dict_attributes_type', 'Input should be a valid dictionary or instance to extract fields from', None),
     ('missing', 'Field required', None),
-    ('frozen', 'Field is frozen', None),
+    ('frozen_field', 'Field is frozen', None),
+    ('frozen_instance', 'Instance is frozen', None),
     ('extra_forbidden', 'Extra inputs are not permitted', None),
     ('invalid_key', 'Keys should be strings', None),
     ('get_attribute_error', 'Error extracting attribute: foo', {'error': 'foo'}),
@@ -397,7 +398,6 @@ class BadRepr:
 
 
 def test_error_on_repr():
-
     s = SchemaValidator({'type': 'int'})
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python(BadRepr())
