@@ -1,4 +1,6 @@
 import re
+from copy import deepcopy
+from typing import Any, Callable, Dict, List, Set, Tuple
 
 import pytest
 
@@ -81,38 +83,126 @@ def test_model_class_setattr():
     )
     m = v.validate_python({'field_a': 'test'})
     assert isinstance(m, MyModel)
-    # assert m.field_a == 'test'
-    # assert m.__fields_set__ == {'field_a'}
+    assert m.field_a == 'test'
+    assert m.__fields_set__ == {'field_a'}
     assert setattr_calls == []
 
 
-def test_model_class_root_validator():
+def test_model_class_root_validator_wrap():
     class MyModel:
-        pass
+        def __init__(self, **kwargs: Any) -> None:
+            self.__dict__.update(kwargs)
 
-    def f(input_value, validator, info):
+    def f(
+        input_value: Dict[str, Any],
+        validator: Callable[[Dict[str, Any]], Dict[str, Any]],
+        info: core_schema.ValidationInfo,
+    ):
+        assert input_value['field_a'] == 123
         output = validator(input_value)
-        return str(output)
+        return output
 
-    v = SchemaValidator(
-        {
-            'type': 'function-wrap',
-            'function': {'type': 'general', 'function': f},
-            'schema': {
-                'type': 'model',
-                'cls': MyModel,
-                'schema': {
-                    'type': 'typed-dict',
-                    'return_fields_set': True,
-                    'fields': {'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'str'}}},
-                },
-            },
-        }
+    schema = core_schema.model_schema(
+        MyModel,
+        core_schema.general_wrap_validator_function(
+            f,
+            core_schema.typed_dict_schema(
+                {'field_a': core_schema.typed_dict_field(core_schema.int_schema())}, return_fields_set=True
+            ),
+        ),
     )
-    assert 'expect_fields_set:true' in plain_repr(v)
-    m = v.validate_python({'field_a': 'test'})
-    assert isinstance(m, str)
-    assert 'test_model_class_root_validator.<locals>.MyModel' in m
+
+    v = SchemaValidator(schema)
+    m = v.validate_python({'field_a': 123})
+    assert m.field_a == 123
+
+    with pytest.raises(ValidationError) as e:
+        v.validate_python({'field_a': 456})
+
+    assert e.value.errors() == [
+        {
+            'type': 'assertion_error',
+            'loc': (),
+            'msg': 'Assertion failed, assert 456 == 123',
+            'input': {'field_a': 456},
+            'ctx': {'error': 'assert 456 == 123'},
+        }
+    ]
+
+
+def test_model_class_root_validator_before():
+    class MyModel:
+        def __init__(self, **kwargs: Any) -> None:
+            self.__dict__.update(kwargs)
+
+    def f(input_value: Dict[str, Any], info: core_schema.ValidationInfo):
+        assert input_value['field_a'] == 123
+        return input_value
+
+    schema = core_schema.model_schema(
+        MyModel,
+        core_schema.general_before_validator_function(
+            f,
+            core_schema.typed_dict_schema(
+                {'field_a': core_schema.typed_dict_field(core_schema.int_schema())}, return_fields_set=True
+            ),
+        ),
+    )
+
+    v = SchemaValidator(schema)
+    m = v.validate_python({'field_a': 123})
+    assert m.field_a == 123
+
+    with pytest.raises(ValidationError) as e:
+        v.validate_python({'field_a': 456})
+
+    assert e.value.errors() == [
+        {
+            'type': 'assertion_error',
+            'loc': (),
+            'msg': 'Assertion failed, assert 456 == 123',
+            'input': {'field_a': 456},
+            'ctx': {'error': 'assert 456 == 123'},
+        }
+    ]
+
+
+def test_model_class_root_validator_after():
+    class MyModel:
+        def __init__(self, **kwargs: Any) -> None:
+            self.__dict__.update(kwargs)
+
+    def f(input_value_and_fields_set: Tuple[Dict[str, Any], Set[str]], info: core_schema.ValidationInfo):
+        input_value, _ = input_value_and_fields_set
+        assert input_value['field_a'] == 123
+        return input_value_and_fields_set
+
+    schema = core_schema.model_schema(
+        MyModel,
+        core_schema.general_after_validator_function(
+            f,
+            core_schema.typed_dict_schema(
+                {'field_a': core_schema.typed_dict_field(core_schema.int_schema())}, return_fields_set=True
+            ),
+        ),
+    )
+
+    v = SchemaValidator(schema)
+    m = v.validate_python({'field_a': 123})
+    assert m.field_a == 123
+
+    with pytest.raises(ValidationError) as e:
+        v.validate_python({'field_a': 456})
+
+    assert e.value.errors() == [
+        {
+            'type': 'assertion_error',
+            'loc': (),
+            'msg': 'Assertion failed, assert 456 == 123',
+            'input': {'field_a': 456},
+            'ctx': {'error': 'assert 456 == 123'},
+        }
+    ]
 
 
 @pytest.mark.parametrize('mode', ['before', 'after', 'wrap'])
@@ -768,7 +858,7 @@ def test_validate_assignment_function():
         field_b: int
         field_c: int
 
-    calls = []
+    calls: List[Any] = []
 
     def func(x, info):
         calls.append(str(info))
@@ -837,7 +927,7 @@ def test_validate_assignment_no_fields_set():
     assert not hasattr(m, '__fields_set__')
 
     # wrong arguments
-    with pytest.raises(TypeError, match='self_instance should not be None on typed-dict validate_assignment'):
+    with pytest.raises(TypeError, match="'field_a' is not a model instance"):
         v.validate_assignment('field_a', 'field_a', b'different')
 
 
@@ -863,3 +953,83 @@ def test_frozen():
     assert exc_info.value.errors() == [
         {'type': 'frozen_instance', 'loc': (), 'msg': 'Instance is frozen', 'input': 'y'}
     ]
+
+
+@pytest.mark.parametrize(
+    'function_schema,call1, call2',
+    [
+        (
+            core_schema.general_after_validator_function,
+            (({'a': 1, 'b': 2}, {'b'}), 'ValidationInfo(config=None, context=None)'),
+            (({'a': 10, 'b': 2}, {'a'}), 'ValidationInfo(config=None, context=None)'),
+        ),
+        (
+            core_schema.general_before_validator_function,
+            ({'b': 2}, 'ValidationInfo(config=None, context=None)'),
+            ({'a': 10, 'b': 2}, 'ValidationInfo(config=None, context=None)'),
+        ),
+        (
+            core_schema.general_wrap_validator_function,
+            ({'b': 2}, 'ValidationInfo(config=None, context=None)'),
+            ({'a': 10, 'b': 2}, 'ValidationInfo(config=None, context=None)'),
+        ),
+    ],
+)
+def test_validate_assignment_model_validator_function(function_schema: Any, call1: Any, call2: Any):
+    """
+    Test handling of values and fields_set for validator functions that wrap a model when using
+    validate_assignment.
+
+    Note that we are currently not exposing this functionality in conjunction with getting
+    access to `fields_set` in a model validator, so the behavior of fields set.
+    In particular, for function_after it is not clear if the fields set passed to
+    the validator should be the fields that were assigned on this call to `validate_assignment`
+    (currently always a single field) or the fields that have been assigned in the
+    model since it was created.
+    """
+
+    class Model:
+        __slots__ = ('__dict__', '__fields_set__')
+
+    calls: List[Any] = []
+
+    def f(values_or_values_and_fields_set: Any, *args: Any) -> Any:
+        if len(args) == 2:
+            # wrap
+            handler, info = args
+            calls.append((deepcopy(values_or_values_and_fields_set), str(info)))
+            return handler(values_or_values_and_fields_set)
+        else:
+            info = args[0]
+            calls.append((deepcopy(values_or_values_and_fields_set), str(info)))
+            return values_or_values_and_fields_set
+
+    v = SchemaValidator(
+        core_schema.model_schema(
+            Model,
+            function_schema(
+                f,
+                core_schema.typed_dict_schema(
+                    {
+                        'a': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(core_schema.int_schema(), default=1)
+                        ),
+                        'b': core_schema.typed_dict_field(core_schema.int_schema()),
+                    },
+                    return_fields_set=True,
+                ),
+            ),
+        )
+    )
+
+    m = v.validate_python({'b': 2})
+    assert m.a == 1
+    assert m.b == 2
+    assert m.__fields_set__ == {'b'}
+    assert calls == [call1]
+
+    v.validate_assignment(m, 'a', 10)
+    assert m.a == 10
+    assert m.b == 2
+    assert m.__fields_set__ == {'a', 'b'}
+    assert calls == [call1, call2]
